/*
 * Author: Hiroyuki Chishiro
 * License: 2-Clause BSD
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <unistd.h>
#include <arpa/inet.h>

#define PORT 65432
#define MAX_SEQ_ID 10
#define BUFSIZE 128
#define ACK "ACK"

int main(void)
{
  int sock;
  int i, n;
  struct sockaddr_in client;
  char buf[BUFSIZE];

  if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
    perror("socket");
    exit(1);
  }

  memset(&client, 0, sizeof(client));
  client.sin_family = AF_INET;
  client.sin_port = htons(PORT);
  client.sin_addr.s_addr = inet_addr("127.0.0.1");

  if (connect(sock, (struct sockaddr *) &client, sizeof(client)) == -1) {
    perror("connect");
    goto end;
  }

  for (i = 0; i <= MAX_SEQ_ID; i++) {
    memset(buf, 0, sizeof(buf));
    sprintf(buf, "%d", i);
    printf("Message to server: %s\n", buf);

    if (write(sock, buf, strlen(buf)) == -1) {
      perror("write");
      goto end;
    }

    if ((n = read(sock, buf, sizeof(buf))) == -1) {
      perror("read");
      goto end;
    }

    buf[n] = '\0';

    printf("Message from server: %s\n", buf);

    if (strcmp(buf, ACK) != 0) {
      fprintf(stderr, "Error: message is not ACK\n");
      goto end;
    }

    usleep(1000);
  }

end:

  if (close(sock) == -1) {
    perror("close");
    exit(2);
  }

  return 0;
}
