/*
 * Author: Hiroyuki Chishiro
 * License: 2-Clause BSD
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <unistd.h>
#include <arpa/inet.h>

#define PORT 65432
#define MAX_SEQ_ID 10
#define BACKLOG 1024
#define BUFSIZE 128
#define ACK "ACK"

int main(void)
{
  int s_sock, c_sock;
  struct sockaddr_in server, other;
  socklen_t add;
  char buf[BUFSIZE];
  int i, n;

  if ((s_sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
    perror("socket");
    exit(1);
  }

  memset(&server, 0, sizeof(server));
  memset(&other, 0, sizeof(other));
  server.sin_family = AF_INET;
  server.sin_port = htons(PORT);
  server.sin_addr.s_addr = INADDR_ANY;

  if (bind(s_sock, (struct sockaddr *) &server, sizeof(server)) == -1) {
    perror("bind");
    goto end;
  }

  if (listen(s_sock, BACKLOG) == -1) {
    perror("listen");
    goto end;
  }

  add = sizeof(other);

  if ((c_sock = accept(s_sock, (struct sockaddr *) &other, &add)) == -1) {
    perror("accept");
    goto end;
  }

  for (i = 0; i <= MAX_SEQ_ID; i++) {
    memset(buf, 0, sizeof(buf));

    if ((n = read(c_sock, buf, sizeof(buf))) == -1) {
      perror("read");
      goto end2;
    }

    printf("Message from client: %s\n", buf);

    if (write(c_sock, ACK, sizeof(ACK)) == -1) {
      perror("write");
      goto end2;
    }

    usleep(1000);
  }

end2:

  if (close(c_sock) == -1) {
    perror("close");
    exit(2);
  }

end:

  if (close(s_sock) == -1) {
    perror("close");
    exit(3);
  }

  return 0;
}
